import requests
import threading
import time
import json
import random
from unsafe.utils.strings import wp_plugins
from unsafe.exceptions import NotWordpress, SiteNotFound, UsersFileNotFound, NotFoundData
from queue import Queue
from typing import Optional

plugins_list = []
counter = 0

queue = Queue()


class Wordpress:

    def __init__(self):
        ...

    def _wp_checker(self, domain: str):
        if 'http://' in domain:
            domain = domain.replace('http://', '')
        elif 'https://' in domain:
            domain = domain.replace('https://', '')
        else:
            domain = domain
        try:
            t = requests.get("http://"+domain)
            if '/wp-content/' in t.text or '/wp-contents/' in t.text or '/wp-includes/' in t.text:
                return True
            s = requests.get("http://"+domain+"/wp-content/plugins/")
            if s.history:
                return False
        except requests.exceptions.ConnectionError:
            return False
        if s.status_code == 404 or s.status_code == 500 or 299 < s.status_code < 305:
            return False
        elif s.status_code == 403 or 199 < s.status_code < 299:
            return True
        return True

    def wp_get_users(self, domain: str):
        try:
            if 'http://' in domain:
                domain = domain.replace('http://', '')
            elif 'https://' in domain:
                domain = domain.replace('https://', '')
            else:
                domain = domain
            if not self._wp_checker(domain):
                raise NotWordpress(domain)
        except:
            ...
        try:
            r = requests.get("https://"+domain+'/wp-json/wp/v2/users/').text
            j = json.loads(r)
            Count = len(j) - 1
            users_list = []
            cn = 0
            User = ''
            for Val in j:
                Split = '\n'
                if Count == cn:
                    Split = ''
                U = j[cn]['slug']
                if not U == '':
                    User += U+Split
                cn += 1
            if User == '':
                raise NotFoundData
            return User.split()
        except json.decoder.JSONDecodeError:
            raise NotWordpress(domain)
        except requests.exceptions.ConnectionError:
            raise SiteNotFound(domain)
        except KeyError:
            raise UsersFileNotFound
        except Exception as e:
            raise e

    def _send_request(self, plugins: list, url: str, timeout: int, proxy: Optional[list | str] = None):
        global plugins_list
        global counter

        if proxy and type(proxy) == list:
            prx = random.choice(proxy)
            proxies = {"https": f"{prx}", "http": f"{prx}"}
        elif proxy and type(proxy) == str:
            proxies = {"https": f"{proxy}", "http": f"{proxy}"}
        else:
            proxies = {}
        for plus in plugins:
            counter += 1
            try:
                if proxies == {}:
                    r = requests.get(
                        f"http://{url}/wp-content/plugins/{plus}/", timeout=timeout)
                else:
                    r = requests.get(
                        f"http://{url}/wp-content/plugins/{plus}/", timeout=timeout, proxies=proxies)
                if r.status_code == 200:
                    plugins_list.append(r.url)
                else:
                    ...
            except (requests.exceptions.ProxyError, requests.exceptions.ConnectTimeout):
                ...
            except Exception as e:
                raise e

    def wp_plugin_scanner(self, domain: str, timeout: int = 3, workers: int = 3, proxy: Optional[list | str] = None):
        if not self._wp_checker(domain):
            raise NotWordpress(domain)

        global queue

        threads = []
        for i in wp_plugins:
            queue.put(i)

        if 'http://' in domain:
            domain = domain.replace('http://', '')
        elif 'https://' in domain:
            domain = domain.replace('https://', '')
        else:
            domain = domain
        for i in range(workers):
            thread_ = threading.Thread(target=self._send_request, args=(
                wp_plugins, domain, timeout, proxy))
            thread_.setDaemon(True)
            thread_.start()
            threads.append(thread_)
        for i in threads:
            i.join()

        global plugins_list
        return plugins_list



"""Utility functions."""

from exif._constants import ATTRIBUTE_ID_MAP
from exif._datatypes import ExifType


def pack_into(datatype, buffer, offset=0):
    """Pack datatype bytes into buffer."""
    buffer[offset : offset + datatype.nbytes] = datatype.ipack()


def value_fits_in_ifd_tag(tag_dt):
    """Determine if value fits inside the IFD tag itself.

    :param Structure tag_dt: IFD tag datatype instance
    :returns: value fits inside IFD tag
    :rtype: bool

    """
    is_value_in_ifd_tag_itself = (
        tag_dt.type == ExifType.ASCII and tag_dt.value_count <= 4
    )
    is_value_in_ifd_tag_itself |= tag_dt.type == ExifType.BYTE
    is_value_in_ifd_tag_itself |= tag_dt.type == ExifType.SHORT
    is_value_in_ifd_tag_itself |= tag_dt.type == ExifType.SSHORT
    is_value_in_ifd_tag_itself |= tag_dt.type == ExifType.LONG
    is_value_in_ifd_tag_itself |= tag_dt.type == ExifType.SLONG
    is_value_in_ifd_tag_itself |= tag_dt.tag_id == ATTRIBUTE_ID_MAP["exif_version"]
    is_value_in_ifd_tag_itself |= tag_dt.tag_id == ATTRIBUTE_ID_MAP["flashpix_version"]

    return is_value_in_ifd_tag_itself



"""IFD SHORT tag structure parser module."""

from plum.bigendian import uint16
from plum.littleendian import uint16 as uint16_le

from exif._constants import (
    ATTRIBUTE_ID_MAP,
    ColorSpace,
    ExposureMode,
    ExposureProgram,
    LightSource,
    MeteringMode,
    Orientation,
    ResolutionUnit,
    Saturation,
    SceneCaptureType,
    SensingMethod,
    Sharpness,
    WhiteBalance,
)
from exif._datatypes import Flash, TiffByteOrder
from exif.ifd_tag._base import Base as BaseIfdTag


class Short(BaseIfdTag):

    """IFD SHORT tag structure parser class."""

    CUSTOM_TYPES_MAP = {
        ATTRIBUTE_ID_MAP["flash"]: Flash,
    }

    ENUMS_MAP = {
        ATTRIBUTE_ID_MAP["color_space"]: ColorSpace,
        ATTRIBUTE_ID_MAP["exposure_mode"]: ExposureMode,
        ATTRIBUTE_ID_MAP["exposure_program"]: ExposureProgram,
        ATTRIBUTE_ID_MAP["metering_mode"]: MeteringMode,
        ATTRIBUTE_ID_MAP["light_source"]: LightSource,
        ATTRIBUTE_ID_MAP["orientation"]: Orientation,
        ATTRIBUTE_ID_MAP["resolution_unit"]: ResolutionUnit,
        ATTRIBUTE_ID_MAP["saturation"]: Saturation,
        ATTRIBUTE_ID_MAP["scene_capture_type"]: SceneCaptureType,
        ATTRIBUTE_ID_MAP["sensing_method"]: SensingMethod,
        ATTRIBUTE_ID_MAP["sharpness"]: Sharpness,
        ATTRIBUTE_ID_MAP["white_balance"]: WhiteBalance,
    }

    def __init__(self, tag_offset, app1_ref):
        super().__init__(tag_offset, app1_ref)

        if self._app1_ref.endianness == TiffByteOrder.BIG:
            self._uint16_cls = uint16
        else:
            self._uint16_cls = uint16_le

    def modify(self, value):
        """Modify tag value.

        This method does not contain logic for unpacking multiple values since the EXIF standard (v2.2) does not list
        any IFD tags of SHORT type with a count greater than 1.

        :param value: new tag value
        :type value: corresponding Python type

        """
        self._uint16_cls.view(
            self._app1_ref.body_bytes, self.tag_view.value_offset.__offset__
        ).set(int(value))

    def read(self):
        """Read tag value.

        This method does not contain logic for unpacking multiple values since the EXIF standard (v2.2) does not list
        any IFD tags of SHORT type with a count greater than 1.

        :returns: tag value
        :rtype: corresponding Python type

        """
        as_int = int(
            self._uint16_cls.view(
                self._app1_ref.body_bytes, self.tag_view.value_offset.__offset__
            )
        )

        try:
            enum_type = self.ENUMS_MAP[int(self.tag_view.tag_id)]
        except KeyError:
            try:
                custom_type = self.CUSTOM_TYPES_MAP[int(self.tag_view.tag_id)]
            except KeyError:
                retval = as_int  # leave return value as-is
            else:
                retval = custom_type.unpack(bytes([as_int]))
        else:
            retval = enum_type(as_int)

        return retval



"""Base IFD tag structure parser module."""

from exif._datatypes import IfdTag, IfdTagLe, TiffByteOrder


class Base:

    """Base IFD tag structure parser class."""

    def __init__(self, tag_offset, app1_ref):
        self._tag_offset = tag_offset
        self._app1_ref = app1_ref

        if self._app1_ref.endianness == TiffByteOrder.BIG:
            self._ifd_tag_cls = IfdTag
        else:
            self._ifd_tag_cls = IfdTagLe

        self.tag_view = self._ifd_tag_cls.view(
            self._app1_ref.body_bytes, self._tag_offset
        )

    def __repr__(self):  # pragma: no cover
        return f"exif.ifd_tag.Base(tag_offset={self._tag_offset})"

    def modify(self, value):  # pragma: no cover
        """Modify tag value.

        :param value: new tag value
        :type value: corresponding Python type

        """
        raise NotImplementedError("cannot modify a base/unknown IFD tag instance")

    def read(self):  # pragma: no cover
        """Read tag value.

        :returns: tag value
        :rtype: corresponding Python type

        """
        raise NotImplementedError("cannot read a base/unknown IFD tag instance")

    def wipe(self):  # pragma: no cover
        """Wipe value pointer target bytes to null."""



from bs4 import BeautifulSoup
import requests
import socks

TIMEOUT = (3.05, 27)
URL = "http://google.com"


class Proxy:
    """Proxy"""

    def __init__(self) -> None:
        ...

    def proxy_wrapper(self, protocol: str = "http", max_ping: int = 200) -> dict:
        """
        ### This Method Crawl a Site For Get Proxie's

        it Takes The Following Arguments:

        * protocol : Proxy Protocol. (Default=http)
        * max_ping : Maximum Ping of Proxy. (Default=200)
        * Available Protocol's : http, socks4, socks5

        * #### return: dictionary
        
        ```
        proxy = Proxy()
        proxylist = proxy.wrapper(protocol = "socks4", max_ping = 100)
        ```
        """
        if protocol.lower() in ["http", "socks5", "socks4"]:
            #Send Request
            req = requests.get(
                f'https://www.freeproxy.world/?type={protocol}&anonymity=&country=&speed={max_ping}&port=&page=1')

            #Create Object and Select Parser Type
            soup = BeautifulSoup(req.text, 'html.parser')

            #Find All Proxy IP's and Port's
            ips = soup.find_all('td', class_='show-ip-div')
            ports = soup.find_all('a')

            list_ip = [str(ip.contents[0].replace('\n', '')) for ip in ips]

            list_port = []

            for i in ports:
                i = i.get_text()
                try:
                    int(i)
                    list_port.append(i)
                except:
                    ...

            #Ordering Proxies
            proxy_list = dict(zip(list_ip, list_port))

            return proxy_list
        else:
            raise ValueError("This Protocol Not Exists.")

    def proxy_checker(self, proxy_host: str, proxy_port: str, protocol: str = "http", timeout: int = 2) -> bool:
        """
        ### This Function is to Check if The Proxy is Dead or Not.

        it Takes The Following Arguments:

        * proxy_host: Proxy's IP
        * proxy_port: Proxy's PORT
        * protocol: http, socks4 or socks5 (Default = http)
        * timeout: The Connection's Timeout (Default = 5)
        
        * #### return: boolean
        
        ```
        proxy = Proxy()
        proxy_check = proxy.wrapper(proxy_host = "10.255.36.45", proxy_port = "80", protocol = "socks4", timeout = 2)
        ```
        """
        proxy_status = False
        if (protocol == "http") or (protocol == "https"):
            try:
                requests.get(
                    "http://www.google.com",
                    proxies={protocol: "http://" +
                             str(proxy_host) + ":" + str(proxy_port)},
                    timeout=timeout,
                )
                proxy_status = True
            except:
                ...
        elif protocol == "socks4":
            try:
                socks4_test = socks.socksocket()
                socks4_test.setproxy(
                    socks.PROXY_TYPE_SOCKS4, proxy_host, proxy_port, True)
                socks4_test.settimeout(timeout)
                socks4_test.connect(("www.google.com", 80))
                proxy_status = True
            except:
                ...
        elif protocol == "socks5":
            try:
                socks5_test = socks.socksocket()
                socks5_test.setproxy(
                    socks.PROXY_TYPE_SOCKS5, proxy_host, proxy_port, True)
                socks5_test.settimeout(timeout)
                socks5_test.connect(("www.google.com", 80))
                proxy_status = True
            except:
                ...
        return proxy_status



class UnknownEncoding(Exception):
    """This Error Raising When You Enter an Unavailable Encoding."""

    def __init__(self, encode: str, message: str = 'Your Encode is Not a Valid Encoding.'):
        self.encode = encode
        self.message = message
        super().__init__(self.encode, self.message)

    def __str__(self):
        return f'{self.message}({self.encode})'


class NotWordpress(Exception):
    """This Error Raising When Your Target Website is Not Built with Wordpess."""

    def __init__(self, url: str, message: str = 'Your Target Website is Not Built with Wordpess.'):
        self.url = url
        self.message = message
        super().__init__(self.url, self.message)

    def __str__(self):
        return f'{self.message}({self.url})'


class SiteNotFound(Exception):
    """This Error Raising When The Program cant Connect to Your Target Website."""

    def __init__(self, url: str, message: str = 'The Program cant Connect to Your Target Website or Entered Domain is Not Found.'):
        self.url = url
        self.message = message
        super().__init__(self.url, self.message)

    def __str__(self):
        return f'{self.message}({self.url})'


class UsersFileNotFound(Exception):
    """This Error Raising When cant Find Users File on Site."""

    def __init__(self, message: str = 'cant Find Users File on Site.'):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.message}'


class NotFoundData(Exception):
    """This Error Raising When cant Find any DATA in Your Proccess."""

    def __init__(self, message: str = 'cant Find any DATA in Your Proccess.'):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.message}'



import cv2
from pathlib import Path
import os
import random
import string


class Anonymous:
    def __init__(self):
        ...

    def _string_generator(self, size=7, chars=string.ascii_uppercase + string.digits):
        return ''.join(random.choice(chars) for _ in range(size))

    def anon_picture(self, image_path: Path, casc_path: Path = 'unsafe/utils/front_sideview_face_detect.xml'):
        """
        Detect And Cover Faces for Anonymously.
        """
        if not os.path.exists('./anon_picture_cache'):
            os.mkdir('./anon_picture_cache')
        face_cascade = cv2.CascadeClassifier(casc_path)
        image = cv2.imread(image_path)
        gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        faces = face_cascade.detectMultiScale(
            gray,
            scaleFactor=1.1,
            minNeighbors=5,
            minSize=(30, 30)
        )
        for (x, y, w, h) in faces:
            cv2.rectangle(image, (x, y), (x+w, y+h), (0, 0, 0), -1)
        file_path = f'anon_picture_cache/{self._string_generator()}.jpg'
        cv2.imwrite(file_path, image)
        return file_path

# a = Anonymous()
# print(a.anon_picture('unsafe/utils/testt.jpg'))



"""IFD SSHORT tag structure parser module."""

from plum.bigendian import sint16
from plum.littleendian import sint16 as sint16_le

from exif._datatypes import TiffByteOrder
from exif.ifd_tag._short import Short
from exif.ifd_tag._base import Base as BaseIfdTag


class Sshort(BaseIfdTag):

    """IFD SHORT tag structure parser class."""

    ENUMS_MAP = Short.ENUMS_MAP

    def __init__(self, tag_offset, app1_ref):
        super().__init__(tag_offset, app1_ref)

        if self._app1_ref.endianness == TiffByteOrder.BIG:
            self._int16_cls = sint16
        else:
            self._int16_cls = sint16_le

    def modify(self, value):  # pragma: no cover
        """Modify tag value.


        :param value: new tag value
        :type value: corresponding Python type

        """
        raise NotImplementedError(
            "this package does not yet support setting SSHORT tags since no SSHORT tags "
            "exist in EXIF specification"
        )

    def read(self):
        """Read tag value.

        This method does not contain logic for unpacking multiple values since the EXIF standard (v2.2) does not list
        any IFD tags of SSHORT type with a count greater than 1.

        :returns: tag value
        :rtype: corresponding Python type

        """
        retval = int(
            self._int16_cls.view(
                self._app1_ref.body_bytes, self.tag_view.value_offset.__offset__
            )
        )

        if int(self.tag_view.tag_id) in self.ENUMS_MAP:
            retval = self.ENUMS_MAP[int(self.tag_view.tag_id)](retval)

        return retval



from setuptools import find_packages, setup

long_description = open('README.md', 'r').read()

requirements = [
    'requests<3.0,>=2.25.1',
    'PySocks==1.7.1',
    'beautifulsoup4==4.11.1',
]

setup(
    name='unsafe',
    version='1.2.16',
    author='Ahura Rahmani',
    author_email='ahur4.rahmani@gmail.com',
    maintainer='Masoud Nayebi',
    maintainer_email='mesut@gmail.com',
    license='MIT',
    url='https://github.com/ahur4/unsafe',
    install_requires=requirements,
    keywords=[
        'hack', 'hack module', 'pentest', 'unsafe', 'osint',
        'osint-python', 'hacking-python', 'cryptography', 'vulnerability-scanner', 'vulnerability', 'security',
        'hacker', 'admin-finder', 'wordpress-scanner', 'instagram-osint', 'vulnerability', 'cve', 'proxy-wrapper', 'proxy-checker',
        'ssh-connect', 'hydra', 'cracker', 'sqlmap', 'ahur4', 'mesutfd'
    ],
    description='A practical and optimal library for those interested in Pentest, cryptography,Vulnerability Scanner and ..',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    package_data={
        'face_recognition': ['utils/*.xml']
    },
    python_requires=">=3.8",
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ]
)



import base64
from unsafe.utils.encrypt import Encryptor
from unsafe.utils.strings import wordlist
from typing import Optional, Union

class Decrypter:
    """
    This is abc Class for Text/File Decryption.
    """

    def __init__(self) -> None:
        ...

    def text_decrypt(self, hash: Optional[str | bytes], word: Optional[str | list] = wordlist, encode: Optional[str] = None, hash_method: str = "MD5", **kwargs) -> Union[str, bytes]:
        """
        function to return encrypted string.
        
        args:

            * hash: your hash to decrypt
            
            `text_decrypt(hash = "2fe89d7b42be7dedb4304ac249fbde9c")`
            
            * encode: your choose to encode word (default=None)
            
            `text_decrypt(hash = "2fe89d7b42be7dedb4304ac249fbde9c", encode = "UTF-8")`
            
            * hash_method: your chosen hash method(default=md5)
            
            `text_decrypt(hash = "2fe89d7b42be7dedb4304ac249fbde9c", hash_method = "SHA1")`
            
            `Available Hash Method's : ['MD5', 'SHA1', 'SHA224', 'SHA256', 'SHA384', 'SHA512', 'SHA3-224', 'SHA3-256', 'SHA3-384', 'SHA3-512', 'SHAKE128', 'SHAKE256', 'BASE16', 'BASE32', 'BASE64', 'BASE85', 'ASCII85', 'CAESAR']`

            * count: for 'CAESAR' method(default = 1)
            
            defalut count for shake method's based on len of hash
            
            
            `text_decrypt(hash = "votbgf", hash_method = 'CAESAR', count = 4)`
        """

        hash_method = hash_method.lower()

        if kwargs == {} and hash_method == 'caesar':
            count = 1
        elif kwargs != {} and hash_method == 'caesar':
            if 'count' in kwargs:
                try:
                    count = int(kwargs['count'])
                except:
                    raise ValueError('count must be integer !')
            else:
                raise ValueError(
                    f'The argument(s) {[i for i in kwargs.keys()]} not defined !')

        if kwargs != {} and hash_method != 'caesar':
            raise ValueError('This Hash Method Have Not Kwargs...')

        if type(word) in [int, float, bool, bytes]:
            word = str(word)

        if hash_method in ['shake128', 'shake256']:
            count = len(hash)

        encryptor = Encryptor()

        if type(word) == str:
            if hash_method == 'md5':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "MD5")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="MD5")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha1':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA1")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA1")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha256':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA256")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA256")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha224':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA224")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA224")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha384':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA384")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA384")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha512':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA512")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA512")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha3-224':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA3-224")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA3-224")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha3-256':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA3-256")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA3-256")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha3-384':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA3-384")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA3-384")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'sha3-512':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHA3-512")) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHA3-512")) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'shake128':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHAKE128", count=str(count))) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHAKE128", count=str(count))) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'shake256':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHAKE256", count=str(count))) == hash:
                        return True
                    else:
                        return False
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHAKE256", count=str(count))) == hash:
                        return True
                    else:
                        return False
            elif hash_method == 'base85':
                return base64.b85decode(hash)
            elif hash_method == 'base64':
                return base64.b64decode(hash)
            elif hash_method == 'base32':
                return base64.b32decode(str(hash))
            elif hash_method == 'base16':
                return base64.b16decode(hash)
            elif hash_method == 'ascii85':
                return base64.a85decode(hash)
            elif hash_method == 'caesar':
                if (type(count) is not int) or (count not in range(1, 27)):
                    raise ValueError(
                        "the key must be an integer between: 1 and 26")
                if (not hash) or (len(hash) == 0):
                    raise ValueError("You must provide data")
                abc = "abcdefghijklmnopqrstuvwxyz"
                ABC = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                c = ""
                for i in hash:
                    if i in abc:
                        c += abc[(abc.index(i) - count) % 26]
                    elif i in ABC:
                        c += ABC[(ABC.index(i) - count) % 26]
                    else:
                        c += i
                return c
            else:
                raise ValueError('This Hash Method Not Available !')
        elif type(word) == list:
            if hash_method == 'base85':
                return base64.b85decode(hash)
            elif hash_method == 'base64':
                return base64.b64decode(hash)
            elif hash_method == 'base32':
                return base64.b32decode(str(hash))
            elif hash_method == 'base16':
                return base64.b16decode(hash)
            elif hash_method == 'ascii85':
                return base64.a85decode(hash)
            elif hash_method == 'md5':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "MD5")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="MD5")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha1':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA1")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA1")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha256':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA256")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA256")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha224':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA224")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA224")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha384':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA384")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA384")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha512':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA512")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA512")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha3-224':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA3-224")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA3-224")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha3-256':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA3-256")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA3-256")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha3-384':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA3-384")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA3-384")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'sha3-512':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(i, encode, "SHA3-512")) == hash:
                            return i
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=i, hash_method="SHA3-512")) == hash:
                            return i
                        else:
                            ...
                return ""
            elif hash_method == 'shake128':
                for i in word:
                    if encode:
                        if str(encryptor.text_encrypt(word, encode, "SHAKE128", count=str(count))) == hash:
                            return word
                        else:
                            ...
                    else:
                        if str(encryptor.text_encrypt(words=word, hash_method="SHAKE128", count=str(count))) == hash:
                            return word
                        else:
                            ...
                return ""
            elif hash_method == 'shake256':
                if encode:
                    if str(encryptor.text_encrypt(word, encode, "SHAKE256", count=str(count))) == hash:
                        return word
                    else:
                        return ""
                else:
                    if str(encryptor.text_encrypt(words=word, hash_method="SHAKE256", count=str(count))) == hash:
                        return word
                    else:
                        return ""
            elif hash_method == 'caesar':
                if (type(count) is not int) or (count not in range(1, 27)):
                    raise ValueError(
                        "the key must be an integer between: 1 and 26")
                if (not hash) or (len(hash) == 0):
                    raise ValueError("You must provide data")
                abc = "abcdefghijklmnopqrstuvwxyz"
                ABC = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                hash = ""
                for i in hash:
                    if i in abc:
                        hash += abc[(abc.index(i) - count) % 26]
                    elif i in ABC:
                        hash += ABC[(ABC.index(i) - count) % 26]
                    else:
                        hash += i
                return hash
            else:
                raise ValueError('This Hash Method Not Available !')
        else:
            raise TypeError('"word" Argument Type Must be : str or list')



"""Image EXIF metadata interface module."""

import logging
import os
import warnings
from typing import Any, BinaryIO, Dict, List, Union

from plum.bigendian import uint16

from exif._constants import ATTRIBUTE_ID_MAP, ExifMarkers
from exif._app1_create import generate_empty_app1_bytes
from exif._app1_metadata import App1MetaData

logger = logging.getLogger(__name__)


class Image:

    """Image EXIF metadata interface class.

    :param img_file: image file with EXIF metadata
    :type image_file: str (file path), bytes (already-read contents), or File

    """

    def _parse_segments(self, img_bytes: bytes) -> None:
        cursor = 0

        # Traverse hexadecimal string until EXIF APP1 segment found.
        while img_bytes[cursor : cursor + len(ExifMarkers.APP1)] != ExifMarkers.APP1:
            cursor += len(ExifMarkers.APP1)
            if cursor > len(img_bytes):
                self._has_exif = False
                cursor = 2  # should theoretically go after SOI marker (if adding)
                break

        self._segments["preceding"] = img_bytes[:cursor]
        app1_start_index = cursor

        if self._has_exif:
            # Determine the expected length of the APP1 segment.
            app1_len = uint16.unpack(
                img_bytes[app1_start_index + 2 : app1_start_index + 4]
            )
            cursor += app1_len + 2  # skip APP1 marker and all data

            # If the expected length stops early, keep traversing until another section is found.
            while img_bytes[cursor : cursor + 1] != ExifMarkers.SEG_PREFIX:
                cursor += 1
                # raise IOError("no subsequent EXIF segment found, is this an EXIF-encoded JPEG?")
                if cursor > len(img_bytes):
                    self._has_exif = False
                    break

        if self._has_exif:
            # Instantiate an APP1 segment object to create an EXIF tag interface.
            self._segments["APP1"] = App1MetaData(img_bytes[app1_start_index:cursor])
            self._segments["succeeding"] = img_bytes[cursor:]
        else:
            # Store the remainder of the image so that it can be reconstructed when exporting.
            self._segments["succeeding"] = img_bytes[app1_start_index:]

    def __init__(
        self,
        img_file: Union[BinaryIO, bytes, str],  # pylint: disable=unsubscriptable-object
    ) -> None:
        self._has_exif = True
        self._segments: Dict[
            str, Union[App1MetaData, bytes]  # pylint: disable=unsubscriptable-object
        ] = {}

        if hasattr(img_file, "read"):
            img_bytes = img_file.read()  # type: ignore
        elif isinstance(img_file, bytes):
            img_bytes = img_file
        elif os.path.isfile(img_file):  # type: ignore
            with open(img_file, "rb") as file_descriptor:  # type: ignore
                img_bytes = file_descriptor.read()
        else:  # pragma: no cover
            raise ValueError("expected file object, file path as str, or bytes")

        self._parse_segments(img_bytes)

    def __dir__(self) -> List[str]:
        members = [
            "delete",
            "delete_all",
            "get",
            "get_all",
            "get_file",
            "get_thumbnail",
            "has_exif",
            "list_all",
            "_segments",
        ]

        if self._has_exif:
            assert isinstance(self._segments["APP1"], App1MetaData)
            members += self._segments["APP1"].get_tag_list()

        return members

    def __getattr__(self, item):
        return getattr(self._segments["APP1"], item)

    def __setattr__(self, key, value):
        try:
            ATTRIBUTE_ID_MAP[key.lower()]
        except KeyError:
            super(Image, self).__setattr__(key, value)
        else:
            if not self._has_exif:
                self._segments["APP1"] = App1MetaData(generate_empty_app1_bytes())
                self._has_exif = True

            setattr(self._segments["APP1"], key.lower(), value)

    def __delattr__(self, item):
        try:
            ATTRIBUTE_ID_MAP[item]
        except KeyError:
            super(Image, self).__delattr__(item)
        else:
            delattr(self._segments["APP1"], item)

    def __getitem__(self, item):
        return self.__getattr__(item)

    def __setitem__(self, key, value):
        self.__setattr__(key, value)

    def __delitem__(self, key):
        self.__delattr__(key)

    def delete(self, attribute: str) -> None:
        """Remove the specified attribute from the image.

        :param attribute: image EXIF attribute name

        """
        delattr(self, attribute)

    def delete_all(self) -> None:
        """Remove all EXIF tags from the image."""
        for _ in range(
            2
        ):  # iterate twice to delete thumbnail tags the second time around
            assert isinstance(self._segments["APP1"], App1MetaData)
            for tag in self._segments["APP1"].get_tag_list():
                if not tag in ["_exif_ifd_pointer", "_gps_ifd_pointer", "exif_version"]:
                    try:
                        delattr(self, tag)
                    except AttributeError:
                        warnings.warn("could not delete tag " + tag, RuntimeWarning)

            self._parse_segments(self.get_file())

    def get(self, attribute: str, default: Any = None) -> Any:
        """Return the value of the specified tag.

        If the attribute is not available or set, return the value specified by the ``default``
        keyword argument.

        :param attribute: image EXIF attribute name
        :param default: return value if attribute does not exist
        :returns: tag value if present, ``default`` otherwise
        :rtype: corresponding Python type

        """
        try:
            retval = getattr(self, attribute)
        except (AttributeError, NotImplementedError):
            retval = default

        return retval

    def get_all(self) -> Dict[str, Any]:
        """Return dictionary containing all EXIF tag values keyed by tag name."""
        all_tags = {}

        for tag_name in self.list_all():
            try:
                tag_value = getattr(self, tag_name)
            except Exception:  # pylint: disable=broad-except
                logger.warning("unable to read tag %r", tag_name)
            else:
                all_tags[tag_name] = tag_value

        return all_tags

    def get_file(self) -> bytes:
        """Generate equivalent binary file contents.

        :returns: image binary with EXIF metadata

        """
        assert isinstance(self._segments["preceding"], bytes)
        img_bytes = self._segments["preceding"]

        if self._has_exif:
            assert isinstance(self._segments["APP1"], App1MetaData)
            img_bytes += self._segments["APP1"].get_segment_bytes()

        assert isinstance(self._segments["succeeding"], bytes)
        img_bytes += self._segments["succeeding"]

        return img_bytes

    def get_thumbnail(self) -> bytes:
        """Extract thumbnail binary contained in EXIF metadata.

        :returns: thumbnail binary
        :raises RuntimeError: image does not contain thumbnail

        """
        thumbnail_bytes = None

        try:
            app1_segment = self._segments["APP1"]
        except KeyError:
            pass
        else:
            assert isinstance(app1_segment, App1MetaData)
            thumbnail_bytes = app1_segment.thumbnail_bytes

        if not thumbnail_bytes:
            raise RuntimeError("image does not contain thumbnail")

        return thumbnail_bytes

    @property
    def has_exif(self) -> bool:
        """Report whether or not the image currently has EXIF metadata."""
        return self._has_exif

    def list_all(self) -> List[str]:
        """List all EXIF tags contained in the image."""
        tags_list = []

        if self._has_exif:
            assert isinstance(self._segments["APP1"], App1MetaData)
            tags_list += self._segments["APP1"].get_tag_list(include_unknown=False)

        return tags_list

    def set(self, attribute: str, value) -> None:
        """Set the value of the specified attribute.

        :param attribute: image EXIF attribute name
        :param value: tag value
        :type value: corresponding Python type

        """
        setattr(self, attribute, value)
